"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-

from libs import management
from helpers.encryption import TwoWayEncryption
from helpers.synchronizing import synchronous
from helpers.caching import CachedMethod
from helpers import convertor
from constants import BACKUP_AND_RESTORE_RESULT_HISTORY_TIME
from constants import LOCAL_COMMAND_TIMEOUT, LOCAL_COMMAND_TIMEOUT_ERROR
from constants import ERROR_DESCRIPTION, UNABLE_TO_CONNECT
import uuid, datetime, sqlite3, threading, json
from eventlet.timeout import with_timeout
from libs import base
from distutils.version import LooseVersion
import ipaddr
import socket
import subprocess
import sys


class BackupAndRestoreManager(object):

    def __init__(self, host, port):

        self._host = host
        self._port = port
        self._conn = sqlite3.connect(':memory:')
        self._conn.text_factory = str
        self._encryptions = dict()
        self.initialize_db()
        self.backup_progress_lock = threading.RLock()
        self.restore_init_lock = threading.RLock()
        self.restore_progress_lock = threading.RLock()
        self._restore = {}

    def initialize_db(self):

        c = self._conn.cursor()
        c.execute("""CREATE TABLE backups(
            id TEXT PRIMARY KEY,
            namespace TEXT,
            destination_node_address TEXT,
            username TEXT,
            password TEXT,
            destination_location TEXT,
            sets TEXT,
            only_metadata TEXT,
            progress_file TEXT,
            delete_it INT,
            timestamp TIMESTAMP,
            status TEXT)""")
        self._conn.commit()

    def add_backup(self, backup_id, namespace, destination_node_address, username,
         password, destination_location, sets, only_metadata, progress_file):

        if username != "" and password is not None:
            self._encryptions[backup_id] = TwoWayEncryption()
            username = self._encryptions[backup_id].encrypt(str(username))
            password = self._encryptions[backup_id].encrypt(str(password))

        c = self._conn.cursor()
        c.execute("""INSERT INTO backups VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
            (backup_id, namespace, destination_node_address, username,
            password, destination_location, sets, only_metadata, progress_file, 0, None, None))

    def get_backups(self):

        c = self._conn.cursor()
        c.execute("SELECT * FROM backups")
        return c.fetchall()

    def get_successful_backups(self):

        c = self._conn.cursor()
        c.execute("SELECT destination_node_address, destination_location, namespace, sets, only_metadata from backups where status like ?", ("%Success%", ))
        return c.fetchall()

    def check_if_backup_exists(self, namespace, destination_node_address,
         destination_location, sets, only_metadata):

        c = self._conn.cursor()
        c.execute("SELECT id, count(id) FROM backups WHERE namespace like ? and destination_node_address like ? and destination_location like ? and sets like ? and only_metadata like ? and status not like ? and status not like ? and status not like ?", (namespace, destination_node_address, destination_location, sets, only_metadata, None, "%Success%", "%Failure%"))
        return c.fetchone()

    def delete_backup(self, backup_id):

        c = self._conn.cursor()
        c.execute("DELETE FROM backups WHERE id like ?", (backup_id, ))

    def update_backup_status(self, backup_id, status):

        c = self._conn.cursor()
        c.execute("UPDATE backups SET timestamp = ?, status = ? WHERE id like ?", (datetime.datetime.now().strftime("%Y/%m/%d %H:%M:%S"), status, backup_id))

    def mark_backup_to_delete_it(self, backup_id):

        c = self._conn.cursor()
        c.execute("UPDATE backups SET delete_it = 1, timestamp = ? WHERE id like ?", (datetime.datetime.now().strftime("%Y/%m/%d %H:%M:%S"), backup_id))

    def initiate_backup(self, ansible_present, namespace, destination_node_address, username,
    password, destination_location, node_list, aerospike_build, sets="ALL",
    only_metadata="False", scan_priority="2", terminate_on_change="True", usr=None, passwd=None, pl_password=None):

        backup_check = self.check_if_backup_exists(namespace, destination_node_address,
                                                   destination_location, sets, only_metadata)
        if backup_check[1] != 0:
            return dict(status="Failure", error="Backup already initiated", id=backup_check[0])
        local_cmd = ""
        optional_args = ""
        if sets != "ALL" and sets != "":
            optional_args += "-s %s " % sets
        if only_metadata == "True":
            optional_args += "-x "
        if terminate_on_change == "True":
            optional_args += "-c "
        if scan_priority != "2":
            optional_args += "-f %s " % str(scan_priority)
        if usr and passwd:
            optional_args += "-U %s -P%s" % (usr, pl_password)

        backup_id = str(uuid.uuid4())
        progress_file = "/tmp/backup-" + backup_id + ".log"

        timestamp = datetime.datetime.now().strftime("%Y-%m-%d_%H:%M:%S")
        destination_location = destination_location.rstrip("/") + "/backup_" + timestamp
        # if aerospike_build.startswith('3'):
        if LooseVersion(aerospike_build) > LooseVersion('3.0.0'):
            cmd = "/bin/bash -c 'mkdir -p %s && asbackup -h %s -p %s -d %s -n %s %s &> %s;'" % (destination_location, self._host, self._port, destination_location, namespace, optional_args, progress_file)
            # local_cmd = ['asbackup', '-h', self._host, '-p', self._port,
            #              '-d', destination_location, '-n', namespace] + \
            #     local_cmd
            local_cmd = 'asbackup -h %s -p %s -d %s -n %s ' % (
                self._host, self._port, destination_location, namespace)
            local_cmd += optional_args
        else:
            cmd = "/bin/bash -c 'mkdir -p %s && clbackup2 -h %s -p %s -d %s -n %s %s &> %s;'" % (destination_location, self._host, self._port, destination_location, namespace, optional_args, progress_file)
            # local_cmd = ['clbackup2', '-h', self._host, '-p', self._port,
            #              '-d', destination_location, '-n', namespace] + \
            #     local_cmd
            local_cmd = 'clbackup2 -h %s -p %s -d %s -n %s ' % (
                self._host, self._port, destination_location, namespace)
            local_cmd += optional_args

        try:
            destination_node_address = socket.gethostbyname(
                destination_node_address)
        except socket.gaierror:
            return {'status': 'Failure', 'error': 'Cannot resolve host address'}

        # Restricting user to take backup on cluster node
        if destination_node_address in node_list:
            return {
                'status': 'Failure',
                'error': 'Destination machine cannot be a cluster node'
            }

        # Get self IP if AS is present
        # If destination IP is loopback IP
        is_address_loopback = False
        self_address = base.get_self_address('localhost', self._port)
        if self_address != UNABLE_TO_CONNECT and self_address not in \
                ERROR_DESCRIPTION:
            self_address = base.split_ip_port(self_address)[0]
            # self_address = self_address.split(":")[0]
            if self_address == destination_node_address:
                is_address_loopback = True

        # If IP Address points to local machine
        if ipaddr.IPAddress(destination_node_address).is_loopback or \
                is_address_loopback:
            # If self IP is present in cluster nodes
            if self_address != UNABLE_TO_CONNECT and self_address not in \
                    ERROR_DESCRIPTION:
                self_address = base.split_ip_port(self_address)[0]
                # self_address = self_address.split(":")[0]
                if self_address in node_list:
                    return {
                        'status': 'Failure',
                        'error': 'Destination machine cannot be a cluster node'
                    }

            msg = "\n[INFO] EXECUTING LOCAL INITIATE BACKUP COMMAND"
            msg += "\n[INFO] Directory Path: %s" % destination_location
            msg += "\n[INFO] Host: %s Port: %s Namespace: %s" % (
                self._host, self._port, namespace)
            result = with_timeout(
                LOCAL_COMMAND_TIMEOUT, self.handle_local_initiate_backup,
                destination_location, local_cmd, progress_file,
                timeout_value=dict(
                    status="Failure",
                    error=LOCAL_COMMAND_TIMEOUT_ERROR % username))
            msg += "\n[INFO] Status: %s\n" % result['status']
            sys.stderr.write(msg)
            destination_node_address = '127.0.0.1'
        else:
            if ansible_present is False:
                return {'status': 'Failure', 'error': 'Ansible Not Found'}

            elif not username or not password:
                return {'status': 'Failure',
                        'error': 'Username or Password not provided'}

            result = management.handle_remote_command(
                destination_node_address, username, password, None,
                cmd, background=3600, backup_flag=True, module_name="shell")

        if result['status'] != "Failure":
            self.add_backup(backup_id, namespace, destination_node_address,
                            username, password, destination_location, sets, only_metadata,
                            progress_file)
            result.update(backup_id=backup_id)

        return result

    @CachedMethod()
    @synchronous('backup_progress_lock')
    def get_backup_progress(self, ansible_present, cluster_level_alerts):

        backups = self.get_backups()
        response = dict()

        for backup in backups:
            if backup[9] != 1:
                response[backup[0]] = dict()
                response[backup[0]]['namespace'] = backup[1]
                response[backup[0]]['destination_node_address'] = backup[2]
                response[backup[0]]['destination_location'] = backup[5]
                if backup[11] is not None:
                    response[backup[0]]['progress'] = json.loads(backup[11])
                    timestamp = datetime.datetime.now() - datetime.timedelta(seconds=BACKUP_AND_RESTORE_RESULT_HISTORY_TIME)
                    timestamp = timestamp.strftime("%Y/%m/%d %H:%M:%S")
                    if timestamp >= backup[10]:
                        self.mark_backup_to_delete_it(backup[0])
                else:
                    username = backup[3]
                    password = backup[4]
                    if username != "" and password is not None:
                        username = self._encryptions[backup[0]].decrypt(str(username))
                        password = self._encryptions[backup[0]].decrypt(str(password))

                    # # Get self IP if AS is present
                    # is_address_loopback = False
                    # self_address = base.get_self_address('localhost',
                    #                                      self._port)
                    # if self_address != UNABLE_TO_CONNECT and \
                    #         self_address not in ERROR_DESCRIPTION:
                    #     self_address = base.split_ip_port(self_address)[0]
                    #     # self_address = self_address.split(":")[0]
                    #     if self_address == backup[2]:
                    #         is_address_loopback = True

                    # if ipaddr.IPAddress(backup[2]).is_loopback or \
                    #         is_address_loopback:
                    if ipaddr.IPAddress(backup[2]).is_loopback:
                        # Backup Progress is Local
                        response[backup[0]]['progress'] = with_timeout(
                            LOCAL_COMMAND_TIMEOUT,
                            self.get_local_backup_progress, backup[8],
                            backup[1],
                            timeout_value=dict(
                                status="Failure",
                                error=LOCAL_COMMAND_TIMEOUT_ERROR % username))
                    else:
                        if ansible_present is False:
                            response[backup[0]]['progress'] = {
                                'status': 'Failure', 'error': 'Ansible Not Found'}
                        else:
                            response[backup[0]]['progress'] = management.get_backup_progress(
                                backup[2], username, password, backup[8], backup[1])

                    if response[backup[0]]['progress']['status'] == "Success" or response[backup[0]]['progress']['status'] == "Failure":
                        self.update_backup_status(backup[0], json.dumps(response[backup[0]]['progress']))
                        if backup[0] in list(self._encryptions.keys()):
                            del self._encryptions[backup[0]]
                    if response[backup[0]]['progress']['status'] == "Success":
                        cluster_level_alerts.on_backup_success(backup[1], backup[2],
                        backup[5])
        return response

    def get_successful_backups_initiated_from_amc(self):

        backups = self.get_successful_backups()
        response = dict(successful_backups=[])
        for backup in backups:
            response['successful_backups'].append(dict(destination_node_address=backup[0],
            destination_location=backup[1], namespace=backup[2], sets=backup[3],
            only_metadata=backup[4]))
        return response

    def get_available_backups(self, ansible_present, source_node_address, source_location,
    username, password):

        source_location = source_location.rstrip("/")

        try:
            source_node_address = socket.gethostbyname(
                source_node_address)
        except socket.gaierror:
            return {'status': 'Failure', 'error': 'Cannot resolve host address'}

        # Get self IP if AS is present
        # If Source IP is loopback IP
        # is_address_loopback = False
        # self_address = base.get_self_address('localhost', self._port)
        # if self_address != UNABLE_TO_CONNECT and self_address not in \
        #         ERROR_DESCRIPTION:
        #     self_address = base.split_ip_port(self_address)[0]
        #     # self_address = self_address.split(":")[0]
        #     if self_address == source_node_address:
        #         is_address_loopback = True

        # if ipaddr.IPAddress(source_node_address).is_loopback or \
        #         is_address_loopback:
        if ipaddr.IPAddress(source_node_address).is_loopback:
            p = subprocess.Popen(
                ['ls %s | grep backup_' % source_location],
                stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
            output, err = p.communicate()
            if p.returncode != 0:
                return {'status': 'Failure', 'error': err}
            return {
                'status': 'Success',
                'available_backups': convertor.text_to_list(output, '\n')
            }
        else:
            if ansible_present is False:
                return {'status': 'Success', 'error': 'Ansible Not Found'}

            elif not username or not password:
                return {'status': 'Failure',
                        'error': 'Username or Password not provided'}

            cmd = "/bin/bash -c 'ls %s | grep backup_'" % source_location
            return management.handle_remote_command(source_node_address,
                username, password, None, cmd, background=0, backup_flag=False,
                module_name="shell", available_backups=True)

    @synchronous('restore_init_lock')
    def initiate_restore(self, ansible_present, source_node_address, source_location, username,
    password, aerospike_build, threads="20", namespace="",
    missing_records_only="False", ignore_generation_num="False", usr=None, passwd=None, pl_password=None):

        if self._restore == {}:
            optional_args = ""
            local_cmd = ""
            if threads != "20":
                optional_args += "-t %s " % threads
            if missing_records_only == "True":
                optional_args += "-u "
            if ignore_generation_num == "True":
                optional_args += "-g "
            if namespace != "":
                optional_args += "-n %s " % namespace
            if usr and passwd:
                optional_args += "-U %s -P%s" % (usr, pl_password)
            restore_id = str(uuid.uuid4())
            progress_file = "/tmp/restore-" + restore_id + ".log"
            source_location = source_location.rstrip("/")

            if aerospike_build.startswith('3'):
                cmd = "/bin/bash -c 'asrestore -h %s -p %s -d %s %s &> %s;'" % (self._host, self._port, source_location, optional_args, progress_file)
                # local_cmd = ['asrestore', '-h', self._host, '-p', self._port,
                #              '-d', source_location] + local_cmd
                local_cmd = 'asrestore -h %s -p %s -d %s ' % (
                    self._host, self._port, source_location)
                local_cmd += optional_args
            else:
                cmd = "/bin/bash -c 'clrestore2 -h %s -p %s -d %s %s &> %s;'" % (self._host, self._port, source_location, optional_args, progress_file)
                # local_cmd = ['clrestore2', '-h', self._host, '-p', self._port,
                #              '-d', source_location] + local_cmd
                local_cmd = 'clrestore2 -h %s -p %s -d %s ' % (
                    self._host, self._port, source_location)
                local_cmd += optional_args

            try:
                source_node_address = socket.gethostbyname(
                    source_node_address)
            except socket.gaierror:
                return {'status': 'Failure',
                        'error': 'Cannot resolve host address'}

            # Get self IP if AS is present
            # If Source IP is loopback IP
            is_address_loopback = False
            self_address = base.get_self_address('localhost', self._port)
            if self_address != UNABLE_TO_CONNECT and self_address not in \
                    ERROR_DESCRIPTION:
                self_address = base.split_ip_port(self_address)[0]
                # self_address = self_address.split(":")[0]
                if self_address == source_node_address:
                    is_address_loopback = True

            # If IP Address points to local machine
            # if ipaddr.IPAddress(source_node_address).is_loopback:
            if ipaddr.IPAddress(source_node_address).is_loopback or \
                    is_address_loopback:
                msg = "\n[INFO] EXECUTING LOCAL INITIATE RESTORE COMMAND"
                msg += "\n[INFO] Source Address: %s" % source_node_address
                msg += "\n[INFO] Source Directory: %s" % source_location
                msg += "\n[INFO] Host: %s Port: %s\n" % (self._host, self._port)
                result = with_timeout(
                    LOCAL_COMMAND_TIMEOUT, self.handle_local_initiate_restore,
                    local_cmd, progress_file,
                    timeout_value=dict(
                        status="Failure",
                        error=LOCAL_COMMAND_TIMEOUT_ERROR % username))
                sys.stderr.write(msg)
                source_node_address = '127.0.0.1'
            else:
                if ansible_present is False:
                    return {'status': 'Failure', 'error': 'Ansible Not Found'}

                elif not username or not password:
                    return {'status': 'Failure',
                            'error': 'Username or Password not provided'}

                result = management.handle_remote_command(source_node_address,
                    username, password, None, cmd, background=3600, backup_flag=True,
                    module_name="shell")

            if result['status'] != "Failure":
                self._restore['source_node_address'] = source_node_address
                self._restore['source_location'] = source_location
                self._restore['namespace'] = namespace
                self._restore['progress_file'] = progress_file
                self._restore['timestamp'] = None
                self._restore['result'] = None
                if username != "" and password is not None:
                    self._restore['encryption'] = TwoWayEncryption()
                    username = self._restore['encryption'].encrypt(str(username))
                    password = self._restore['encryption'].encrypt(str(password))
                self._restore['username'] = username
                self._restore['password'] = password
            return result
        return dict(status="Failure", error="A restore operation on this cluster is already in progress")

    @CachedMethod()
    @synchronous('restore_progress_lock')
    def get_restore_progress(self, ansible_present, aerospike_build,
                             cluster_level_alerts):
        if self._restore != {}:
            if self._restore['result'] != None:
                timestamp = datetime.datetime.now() - datetime.timedelta(seconds=BACKUP_AND_RESTORE_RESULT_HISTORY_TIME)
                timestamp = timestamp.strftime("%Y/%m/%d %H:%M:%S")
                result = self._restore['result']
                if timestamp >= self._restore['timestamp']:
                    self._restore = {}
                return result
            username = self._restore['username']
            password = self._restore['password']
            if username != "" and password is not None:
                username = self._restore['encryption'].decrypt(str(username))
                password = self._restore['encryption'].decrypt(str(password))

            # # Get self IP if AS is present
            # is_address_loopback = False
            # self_address = base.get_self_address('localhost', self._port)
            # if self_address != UNABLE_TO_CONNECT and self_address not in \
            #         ERROR_DESCRIPTION:
            #     self_address = base.split_ip_port(self_address)[0]
            #     # self_address = self_address.split(":")[0]
            #     if self_address == self._restore['source_node_address']:
            #         is_address_loopback = True

            # if ipaddr.IPAddress(
            #         self._restore['source_node_address']).is_loopback or \
            #         is_address_loopback:
            if ipaddr.IPAddress(
                    self._restore['source_node_address']).is_loopback:
                # Backup Progress is Local
                result = with_timeout(
                    LOCAL_COMMAND_TIMEOUT,
                    self.get_local_restore_progress,
                    self._restore['progress_file'], aerospike_build,
                    timeout_value=dict(
                        status="Failure",
                        error=LOCAL_COMMAND_TIMEOUT_ERROR % username))
            else:
                if ansible_present is False:
                    return {'status': 'Failure', 'error': 'Ansible Not Found'}

                elif not username or not password:
                    return {'status': 'Failure',
                            'error': 'Username or Password not provided'}

                result = management.get_restore_progress(
                    self._restore['source_node_address'], username, password,
                    self._restore['progress_file'], aerospike_build)

            if result['status'] == "Success":
                cluster_level_alerts.on_restore_success(self._restore['namespace'],
                    self._restore['source_node_address'],
                    self._restore['source_location']
                    )
            if result['status'] == "Success" or result['status'] == "Failure":
                self._restore['result'] = result
                self._restore['timestamp'] = datetime.datetime.now().strftime("%Y/%m/%d %H:%M:%S")
            return result
        return dict(status="")

    @staticmethod
    def handle_local_initiate_backup(destination_location, cmd, progress_file):
        """
        Fire a new subprocess to run AS backup
        :param destination_location: Directory path of backup file
        :param cmd: Command to fire AS Backup
        :param progress_file: Progress file path to write logs
        :return: {'status': <status>, [, 'error': <error>]}
        """
        # CREATE DIRECTORY COMMAND
        p = subprocess.Popen(
            ['mkdir', '-p', destination_location],
            stderr=subprocess.PIPE, cwd='/')
        output = p.communicate()[1]
        if p.returncode != 0:
            return {'status': 'Failure', 'error': output}

        # BACKUP COMMAND
        try:
            f = open(progress_file, 'w')
        except (IOError, Exception):
            return {'status': 'Failure', 'error': 'Failed to open log file'}
        subprocess.Popen(cmd, stderr=f, stdout=f, cwd='/', shell=True)
        f.close()
        return {'status': 'In Progress'}

    @staticmethod
    def get_local_backup_progress(progress_file, namespace):
        """
        Check for AS backup progress on machine of AMC server
        :param progress_file: Log file written during backup progress
        :param namespace: Namespace name
        :return: {'status': <status> [,'error': <error>, 'percentage': <pct>]}
        """
        p = subprocess.Popen(
            ['cat', progress_file], stderr=subprocess.PIPE,
            stdout=subprocess.PIPE, cwd='/')
        output, error = p.communicate()
        if p.returncode != 0:
            return {'status': 'Failure', 'error': error}

        messages = {
            'command not found': 'asbackup not installed',
            'could not get full path for':
                'Invalid Destination Filesystem Path',
            'Running out of disk space': 'Insufficient disk space',
            'could not connect to host': 'Could not connect to node',
            'Aerospike getmany failed':
                'Backup terminated due to cluster change',
            'not accessible or timed out':
                'Backup failed due to cluster change',
            'No objects in cluster or namespace %s doesn\'t exist.' % (
                namespace):
                'No objects in cluster or namespace %s doesn\'t exist' % (
                    namespace)
        }

        for key, value in messages.items():
            if output.find(key) != -1:
                subprocess.Popen(['rm', '-rf', progress_file], cwd='/')
                return {'status': 'Failure', 'error': value}

        if output.find('Backed up') != -1:
            subprocess.Popen(['rm', '-rf', progress_file], cwd='/')
            return {'status': 'Success'}

        if output.find('Backup successfully completed.') != -1:
            subprocess.Popen(['rm', '-rf', progress_file], cwd='/')
            return {'status': 'Success'}

        last_pct = output.rfind("%")
        last_nonspace_before_pct = output.rfind(" ", 0, last_pct)

        if last_pct is -1 or last_nonspace_before_pct is -1:
            if output.find('[ERR]'):
                return {'status': 'Failure',
                        'error': 'Error in Backup process'}
            else:
                return {'status': 'In Progress',
                        'percentage': 'Initiating backup'}
        else:
            pct = output[last_nonspace_before_pct + 1: last_pct + 1]

            space_before_keyword = output.rfind(" ", 0, last_nonspace_before_pct)
            keyword_before_pct = output[
                space_before_keyword + 1: last_nonspace_before_pct]

            if pct == "100%" and keyword_before_pct == 'records':
                subprocess.Popen(['rm', '-rf', progress_file], cwd='/')
                return {'status': 'Success'}
            else:
                return {'status': 'In Progress', 'percentage': pct}

    @staticmethod
    def handle_local_initiate_restore(cmd, progress_file):
        """
        Fire a new subprocess to run AS restore
        :param cmd: Command to fire AS restore on local machine
        :param progress_file: Progress file path to write logs
        :return: {'status': 'In Progress'}
        """
        try:
            f = open(progress_file, 'w')
        except (IOError, Exception):
            return {'status': 'Failure', 'error': 'Failed to open log file'}
        subprocess.Popen(cmd, stderr=f, cwd='/', shell=True)
        f.close()
        return {'status': 'In Progress'}

    @staticmethod
    def get_local_restore_progress(progress_file, aerospike_build):
        """
        Check for AS Restore Progress on machine of AMC server
        :param progress_file: Log file written during restore progress
        :param aerospike_build: Aerospike Build Version
        :return: {'status': <status> [,'error': '<error>']}
        """
        # Check if restore command is present in active processes
        cmd = ['ps aux | grep %s | grep -v grep' % (
            'asrestore' if aerospike_build.startswith('3') else 'clrestore2'
        )]
        p = subprocess.Popen(
            cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)
        output, err = p.communicate()
        # If found then restore command is still running
        if output:
            return {'status': 'In Progress'}

        # Read progress file
        p = subprocess.Popen(
            ['cat', progress_file], stderr=subprocess.PIPE,
            stdout=subprocess.PIPE, cwd='/')
        output, error = p.communicate()
        if p.returncode != 0:
            return {'status': 'Failure', 'error': error}

        # Check for known errors
        messages = {
            'command not found': 'asrestore not installed',
            'could not get full path for': 'Invalid Source Filesystem Path',
            'could not connect to host': 'Could not connect to node',
            'restore failed due to unavailability of namespace in backup ' + (
                'file: suspecting wrong data format in backup files'):
                    'Namespace unavailable in backup file: suspecting wrong' + (
                        ' data format in backup files'),
            'restore failed due to wrong namespace name':
                'Restore failed due to wrong namespace name'
        }
        for key, value in messages.items():
            if output.find(key) != -1:
                subprocess.Popen(['rm', '-rf', progress_file], cwd='/')
                return {'status': 'Failure', 'error': value}

        if output.find("Starting restore"):
            if output.find("100% complete"):
                result = {'status': 'Success'}
            else:
                result = {'status': 'Failure', 'error': 'Restore unsuccessful'}
        else:
            num_of_backup_files = output.count("starting restore: filename:")
            if num_of_backup_files == 0:
                result = {'status': 'Failure', 'error': 'No backup files found'}
            else:
                num_of_successful_restores = output.count("restore complete")
                if num_of_backup_files == num_of_successful_restores:
                    result = {'status': 'Success'}
                else:
                    result = {'status': 'Failure', 'error': 'Restore unsuccessful'}

        subprocess.Popen(['rm', '-rf', progress_file], cwd='/')
        return result
